/* Purpose: Eight queens problem solver 
 * Author:  Serge Aleynikov <serge@hq.idt.net>
 * Date:    30-Dec-2006
 * Compile: gcc -Wall 8queens.c -o queens_c
 */
#include <stdio.h>
#include <stdlib.h>

static int* board;
static int  sol_count = 0;
static int  aborted   = 0;
typedef int bool;

/* Solution output */

void output(int dim, int max_sol, bool display) 
{
    int i, j;
    
    sol_count++;
    if (!aborted && display) {
        printf("%-8d/", sol_count);
        for (i=1; i <= dim; ++i) printf("%2d", i);
        printf("\n");
        for (i=1; i <= dim; ++i) {
            printf("%8d: ", i);
            for (j=0; j < dim; ++j) {
                printf("%s ", (board[i-1] == j) ? "Q" : "*");
            }
            printf("\n");
        }
        printf("\n");
    }
    aborted = max_sol && sol_count >= max_sol;
}

/* Tells if row i not threatened */

bool safe(int dim, int i) 
{
    int j;
    
    for (j=0; j < dim; ++j)
        if (board[i] == board[j] || abs(board[i] - board[j]) == i-j)
            return i == j;
    return i == dim-1;
}

/* Main algorithm called recursively */

void queens(int dim, int max_sol, bool display, int i)
{
    int j;
    
    for(j = 0; j < dim && !aborted; ++j) {
        board[i] = j;
        if (safe(dim, i)) {
            if (i == dim-1)
                output(dim, max_sol, display);
            else
                queens(dim, max_sol, display, i+1);
        }
    }
}

int main(int argc, char* argv[])
{
   int dim, times, max_sol, i;
   bool display;
   
   if (argc == 1) {
       printf("Usage: %s dimension [iterations] [max_solutions]\n", argv[0]);
       return 1;
   }
   
   dim     = atoi(argv[1]);
   times   = (argc > 2) ? atoi(argv[2]) : 1;
   max_sol = (argc > 3) ? atoi(argv[3]) : 0; 
   display = times == 1;

   board   = (int*) calloc(dim, sizeof(int));
   
   for (i=0; !aborted && i < times; ++i) {
       sol_count=0;
       queens(dim, max_sol, display, 0);
       if (!i)
           printf("Total: %d solutions\n", sol_count);
   }

   free(board);

   return 0;
}
