/* Purpose: N-queens problem solver
 * Author:  Serge Aleynikov <serge@hq.idt.net>
 * Date:    30-Dec-2006
 * Compile: g++ -Wall nqueens.cpp -o queens_cpp
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdexcept>

class Abort: public std::exception {};

class NQueens
{
    public:
        NQueens(int Dim, int Max_sol, int Display): 
            dim(Dim), display(Display), sol_count(0), 
            max_sol(Max_sol), board(new int[Dim]) {}

        ~NQueens() { delete [] board; }

        void Queens(int i);
        void ResetSolCount() { sol_count = 0; }
        int  SolCount()      { return sol_count; }
    private:
        int  dim;
        bool display;
        int  sol_count;
        int  max_sol;
        int* board;

        bool Safe(int i);
        void Output() throw(Abort);
};

/* Solution output */

void NQueens::Output() throw(Abort)
{
    sol_count++;
    if (display) {
        printf("%-8d/", sol_count);
        for (int i=1; i <= dim; ++i) printf("%2d", i);
        printf("\n");
        for (int i=1; i <= dim; ++i) {
            printf("%8d: ", i);
            for (int j=0; j < dim; ++j) {
                printf("%s ", (board[i-1] == j) ? "Q" : "*");
            }
            printf("\n");
        }
        printf("\n");
    }
    if (max_sol && sol_count >= max_sol)
        throw Abort();
}

/* Tells if row i is not threatened */

bool NQueens::Safe(int i) 
{
    for (int j=0; j < dim; ++j)
        if (board[i] == board[j] || abs(board[i] - board[j]) == i-j)
            return i == j;
    return i == dim-1;
}

/* Main algorithm called recursively */

void NQueens::Queens(int i)
{
    for(int j = 0; j < dim; ++j) {
        board[i] = j;
        if (Safe(i)) {
            if (i == dim-1)
                Output();
            else
                Queens(i+1);
        }
    }
}

int main(int argc, char* argv[])
{
    int dim, times, max_sol;
    bool display;

    if (argc == 1) {
        printf("Usage: %s dimension [iterations] [max_solutions]\n", argv[0]);
        return 1;
    }

    dim     = atoi(argv[1]);
    times   = (argc > 2) ? atoi(argv[2]) : 1;
    max_sol = (argc > 3) ? atoi(argv[3]) : 0; 
    display = times == 1;

    NQueens queens(dim, max_sol, display);
   
    try {
        for (int i=0; i < times; ++i) {
            queens.ResetSolCount();
            queens.Queens(0);
            if (!i)
                printf("Total: %d solutions\n", queens.SolCount());
        }
    } catch (Abort &e) {}
    
    return 0;
}
