%% Purpose: Eight queens problem solver 
%% Author:  Serge Aleynikov <serge@hq.idt.net>
%% Date:    30-Dec-2006
%% Compile: erlc +native nqueens.erl
%% Run:     erl -noshell -run nqueens main 8 1000 -s erlang halt
%%
-module(nqueens).
-author('serge@hq.idt.net').
-export([main/1, queens/3]).

loop(N, N, F, A) -> F(N, A);
loop(I, N, F, A) -> A1 = F(I, A), loop(I+1, N, F, A1).
loop_nl(I, N, F) -> loop(I, N, F, ok), io:nl().

%% Output solution to console if Display is true. 
%% Throws 'abort' when counter signals that MaxCount is reached.
output(_Board, _SolCount, false = _Display) -> ok;
output( Board,  SolCount, _Display) ->
    io:format("~-8w/", [SolCount]),
    N = size(Board),
    loop_nl(1, N, fun(I,_) -> io:format("~2w"  , [I]) end),
    loop_nl(1, N, fun(I,_) -> io:format("~8w: ", [I]), 
                              QI = element(I, Board),
                              loop_nl(1, N, fun(J,_) when J==QI -> io:put_chars("Q "); 
                                               (_,_)            -> io:put_chars("* ") 
                                            end) 
                  end).
    
% Is the queen at row I not threatened
safe(_Brd, N, I, J) when J==N -> I==J;                                        % last column
safe( Brd,_N, I, J) when element(I,Brd) == element(J,Brd) -> I==J;            % vert/horiz threat
safe( Brd,_N, I, J) when abs(element(I,Brd)-element(J,Brd)) == (I-J) -> I==J; % diag threat
safe( Brd, N, I, J) -> safe(Brd, N, I, J+1).

% Main algorithm
queens(N, Times, MaxSol) 
  when is_integer(N), is_integer(Times), is_integer(MaxSol), N > 0, Times > 0 ->
    Board = erlang:make_tuple(N, 0),
    F = fun(_,_) -> queens1(Board, MaxSol, 0, Times == 1, 1) end,
    TotalSol = (catch loop(1, Times, F, 0)),
    io:format("Total: ~w solutions~n", [TotalSol]).
        
queens1(Board, MaxSol, SolCount, Display, I) ->
    F = fun(J, Sum) -> queens2(setelement(I, Board, J), MaxSol, Sum, Display, I) end, 
    loop(1, size(Board), F, SolCount).
queens2(Board, MaxSol, SolCount, Display, I) -> 
    N = size(Board),
    case safe(Board, N, I, 1) of
    false            -> SolCount;
    true when I == N -> SC = SolCount+1,
                        output(Board, SC, Display), 
                        SC == MaxSol andalso throw(MaxSol),
                        SC;
    _                -> queens1(Board, MaxSol, SolCount, Display, I+1)
    end.

% Main function

main([Size, Times]) -> main([Size, Times, "0"]);
main([Size, Times, MaxSol]) when is_list(Size), is_list(Times), is_list(MaxSol) ->
    queens(list_to_integer(Size), list_to_integer(Times), list_to_integer(MaxSol)).
