(* Purpose: Eight queens problem solver 
 * Author:  Serge Aleynikov <serge@hq.idt.net>
 * Date:    30-Dec-2006
 * Compile: ocamlopt.opt -unsafe nqueens.ml -o queens_ml
 *)
let sol_count  = ref 0  (* Solution counter *) 

(* Solution printing *)

let output a n max_sol display = 
    incr sol_count;
    if not display then ()
    else ( 
        Printf.printf "%-8d/" !sol_count;
        for i = 1 to n do Printf.printf "%2d" i done;
        print_newline ();
        for i = 1 to n do 
            Printf.printf "%8d: " i;
            for j = 0 to n-1 do 
                print_string (if a.(i-1) = j then "Q " else "* ");
            done;
            print_newline ()
        done;
        print_newline ()
    ); 
    if !sol_count = max_sol then raise Exit else ()

let rec safe a n i = function
    | j when j = n-1                       -> i = j   (* are we on the last column *)
    | j when a.(i) = a.(j)                 -> i = j   (* is there vertical/horizontal threat? *)
    | j when abs (a.(i) - a.(j)) = (i - j) -> i = j   (* is there diagonal threat? *)
    | j                                    -> safe a n i (j+1)

let rec queens a n max_sol display i = 
    for j = 0 to n-1 do 
        a.(i) <- j; 
        match (safe a n i 0) with
        | false             -> ()                               (* this column is threatened *)
        | true when i = n-1 -> output a n max_sol display       (* solution found *)
        | true              -> queens a n max_sol display (i+1) (* check remaining rows *)
    done;;

(* Main program *)

if !Sys.interactive then () 
else let args = (Array.length Sys.argv) - 1 in
    if args < 1 then 
        let _ = Printf.printf "Usage: %s dimension [iterations] [max_solutions]\n" 
                              (Filename.basename Sys.argv.(0)) in
        exit 1
    else
        let dim     = int_of_string Sys.argv.(1) in
        let times   = if args > 1 then int_of_string Sys.argv.(2) else 1 in
        let max_sol = if args > 2 then int_of_string Sys.argv.(3) else 0 in
        let display = (times = 1) in
        for i = 1 to times do
            sol_count := 0;
            try  queens (Array.create dim 0) dim max_sol display 0
            with Exit -> ();
        done;
        Printf.printf "Total: %d solutions\n" !sol_count;;
